@@ -1,4 +1,5 @@
 setClass("Line", 
+	#slots = c(coords = "matrix"),
 	representation(coords = "matrix"),
 	prototype = list(coords = matrix(0)),
 	validity = function(object) {
@@ -13,6 +14,7 @@ setClass("Line",
 )
 
 setClass("Lines",
+	#slots = c(Lines = "list", ID = "character"),
 	representation(Lines = "list", ID = "character"),
 	validity = function(object) {
 		if (any(sapply(object@Lines, function(x) !is(x, "Line"))))
@@ -21,7 +23,9 @@ setClass("Lines",
 })
 
 setClass("SpatialLines",
-	representation("Spatial", lines = "list"),
+	contains = "Spatial", 
+	#slots = c(lines = "list"),
+	representation(lines = "list"),
 	prototype = list(bbox = matrix(rep(NA, 2), 2, 2, 
 			dimnames = list(NULL, c("min","max"))),
 		proj4string = CRS(as.character(NA)),
