@@ -1,6 +1,9 @@
 setClass("Polygon",
-	representation("Line", labpt = "numeric", 
-	area = "numeric", hole = "logical", ringDir = "integer"),
+	contains = "Line", 
+	#slots = c(labpt = "numeric", area = "numeric", hole = "logical", 
+	#	ringDir = "integer"),
+	representation(labpt = "numeric", area = "numeric", hole = "logical", 
+		ringDir = "integer"),
 	validity = function(object) {
                 res <- .Call(Polygon_validate_c, object)
                 res
@@ -16,8 +19,10 @@ setClass("Polygon",
 )
 
 setClass("Polygons",
+	#slots = c(Polygons = "list", plotOrder = "integer", 
+	#	labpt = "numeric", ID = "character", area = "numeric"),
 	representation(Polygons = "list", plotOrder = "integer", 
-	labpt = "numeric", ID = "character", area = "numeric"),
+		labpt = "numeric", ID = "character", area = "numeric"),
 	validity = function(object) {
                 res <- .Call(Polygons_validate_c, object)
                 res
@@ -32,9 +37,9 @@ setClass("Polygons",
 )
 
 setClass("SpatialPolygons",
-	representation("Spatial",
-		polygons = "list",
-		plotOrder = "integer"),
+	contains = "Spatial",
+	#slots = c(polygons = "list", plotOrder = "integer"),
+	representation(polygons = "list", plotOrder = "integer"),
 	prototype = list(
 		bbox = matrix(rep(NA, 4), 2, 2, dimnames = list(NULL, c("min","max"))),
 		proj4string = CRS(as.character(NA)),
