@@ -1,11 +1,12 @@
 SpatialPixels = function(points, tolerance = sqrt(.Machine$double.eps), 
-		proj4string = CRS(as.character(NA)), round=NULL) {
+		proj4string = CRS(as.character(NA)), round = NULL, grid = NULL) {
 	if (!is(points, "SpatialPoints"))
 		stop("points should be of class or extending SpatialPoints")
 	is.gridded = gridded(points)
-#	points = as(points, "SpatialPoints")
-	if (is.na(proj4string(points))) proj4string(points) = proj4string
-	grid = points2grid(points, tolerance, round)
+	if (is.na(proj4string(points))) 
+		proj4string(points) = proj4string
+	if (is.null(grid))
+		grid = points2grid(points, tolerance, round)
 	if (!is.gridded) {
 		points@bbox[,1] = points@bbox[,1] - 0.5 * grid@cellsize
 		points@bbox[,2] = points@bbox[,2] + 0.5 * grid@cellsize
@@ -14,6 +15,26 @@ SpatialPixels = function(points, tolerance = sqrt(.Machine$double.eps),
 		grid.index = getGridIndex(coordinates(points), grid))
 }
 
+SpatialGrid = function(grid, proj4string = CRS(as.character(NA))) {
+	stopifnot(is(grid, "GridTopology"))
+	if (is.character(proj4string))
+		proj4string = CRS(proj4string)
+	cc = rbind(grid@cellcentre.offset - 0.5 * grid@cellsize,
+			grid@cellcentre.offset + (grid@cells.dim - 0.5) * grid@cellsize)
+	bb = .bboxCoords(cc)
+	new("SpatialGrid", grid = grid, bbox = bb, proj4string = proj4string)
+}
+
+setAs("SpatialGrid", "SpatialPixels", 
+	function(from) { 
+		pts = as(from, "SpatialPoints")
+		new("SpatialPixels", pts, grid = from@grid, grid.index = 1:length(pts))
+	}
+)
+
+setAs("SpatialGrid", "SpatialPoints", 
+	function(from) SpatialPoints(coordinates(from), from@proj4string))
+
 setMethod("coordinates", "SpatialPixels", function(obj) obj@coords)
 
 row.names.SpatialPixels <- function(x) {
@@ -23,39 +44,17 @@ row.names.SpatialPixels <- function(x) {
 	ret
 }
 
-row.names.SpatialGrid <- function(x) {
-	#warning("row.names order might not reflect grid sequence!")
-	1:prod(x@grid@cells.dim)
-}
-
-SpatialGrid = function(grid, proj4string = CRS(as.character(NA))) {
-	pts = boguspoints(grid)
-	pts@bbox[,1] = pts@bbox[,1] - 0.5 * grid@cellsize
-	pts@bbox[,2] = pts@bbox[,2] + 0.5 * grid@cellsize
-	proj4string(pts) = proj4string
-# RSB
-#	new("SpatialGrid", pts, grid = grid, grid.index = integer(0))
-#	res <- new("SpatialGrid", new("SpatialPixels", pts, grid = grid, 
-#		grid.index = integer(0)))
-# very odd long initiation times 071006 RSB
-
-	SPix <- new("SpatialPixels", pts, grid = grid, grid.index = integer(0))
-	res <- new("SpatialGrid")
-	slot(res, "grid") <- slot(SPix, "grid")
-	slot(res, "grid.index") <- slot(SPix, "grid.index")
-	slot(res, "coords") <- slot(SPix, "coords")
-	slot(res, "bbox") <- slot(SPix, "bbox")
-	slot(res, "proj4string") <- slot(SPix, "proj4string")
-	res
-# avoids recursive validation in new("SpatialGrid", ...)
-# representation of SG is Spix
-}
+row.names.SpatialGrid <- function(x) 1:prod(x@grid@cells.dim)
 
 setMethod("coordinates", "SpatialGrid", function(obj) coordinates(obj@grid))
 
+setMethod("plot", signature(x = "SpatialGrid", y = "missing"),
+    function(x,y,...) plot(as(x, "SpatialPoints"),...))
+
 coordnamesSG = function(x, value) {
 	dimnames(x@bbox)[[1]] = value
-	dimnames(x@coords)[[2]] = value
+	if (is(x, "SpatialPixels"))
+		dimnames(x@coords)[[2]] = value
 	coordnames(x@grid) = value
 	x
 }
@@ -66,8 +65,8 @@ setReplaceMethod("coordnames",
 	signature(x = "SpatialPixels", value = "character"), coordnamesSG)
 
 getGridTopology = function(obj) {
-	if (!is(obj, "SpatialPixels"))
-		stop("object is not or does not extend class SpatialPixels")
+	if (!(is(obj, "SpatialPixels") || is(obj, "SpatialGrid")))
+		stop("object is not or does not extend class SpatialPixels or SpatialGrid")
 	obj@grid
 }
 
@@ -80,7 +79,7 @@ areaSpatialGrid = function(obj) {
 }
 
 gridparameters = function(obj) { 
-	if (is(obj, "SpatialPixels"))
+	if (is(obj, "SpatialPixels") || is(obj, "SpatialGrid"))
 		obj = obj@grid
 	if (is(obj, "GridTopology"))
 		return(data.frame(
@@ -90,12 +89,6 @@ gridparameters = function(obj) {
 	return(numeric(0))
 }
 
-boguspoints = function(grid) {
-	x = rbind(grid@cellcentre.offset, 
-			grid@cellcentre.offset + (grid@cells.dim - 1) * grid@cellsize)
-	SpatialPoints(x)
-}
-
 getGridIndex = function(cc, grid, all.inside = TRUE) {
 	n = ncol(cc)
 	idx = rep(1, nrow(cc))
@@ -123,42 +116,54 @@ getGridIndex = function(cc, grid, all.inside = TRUE) {
 	as.integer(round(idx))
 }
 
-subset.SpatialPixels <- function(x, subset, select, drop = FALSE, ...) {
-	xSP <- as(x, "SpatialPoints")
-	if (missing(select)) select <- colnames(coordinates(xSP))
-	res <- subset(xSP, subset=subset, select=select, drop = drop, ...)
-	gridded(res) = TRUE
-	res
+rcFromGridIndex = function(obj) {
+	obj = as(obj, "SpatialPixels")
+	gi = obj@grid.index
+	grid = obj@grid
+	stopifnot(ncol(coordinates(obj)) == 2)
+	xi = (gi %% grid@cells.dim[1]) + 1
+	yi = grid@cells.dim[2] - ((gi - 1) %/% grid@cells.dim[1])
+	cbind(xi,yi)
+}
+
+gridIndex2nb = function(obj, maxdist = sqrt(2), fullMat = TRUE, ...) {
+	xy = rcFromGridIndex(obj)
+	if (fullMat)
+		lst = apply(as.matrix(dist(xy, ...)), 1, function(x) which(x <= maxdist))
+	else {
+		dst = function(X, Xi) apply(X, 1, function(Y) sqrt(sum((Y-Xi)^2)))
+		n = nrow(xy)
+		lst = vector(mode = "list", length = n)
+		for (i in 1:n) { # avoid the n x n matrix construction:
+			d = dst(xy, xy[i,])
+			lst[[i]] = which(d <= maxdist)
+		}
+	}
+	class(lst) = c("nb", "list")
+	lst
 }
 
 setMethod("[", "SpatialPixels",
-	function(x, i, j, ..., drop = TRUE) {
-#		if (!missing(drop))
-#			stop("don't supply drop: it needs to be FALSE anyway")
+	function(x, i, j, ..., drop = FALSE) {
 		if (!missing(j))
 			stop("can only select pixels with a single index")
 		if (missing(i))
 			return(x)
 		if (is(i, "Spatial"))
 			i = !is.na(over(x, geometry(i)))
-		if (drop) { # default: adjust bbox and grid
+		if (drop) { # if FALSE: adjust bbox and grid
 			res = as(x, "SpatialPoints")[i]
 			tolerance = list(...)$tolerance
 			if (!is.null(tolerance))
 				res = SpatialPixels(res, tolerance = tolerance)
 			else
 				gridded(res) = TRUE
-		} else {
-			res = new("SpatialPixels", bbox = x@bbox, 
+			res
+		} else # default: don't adjust bbox and grid
+			new("SpatialPixels", bbox = x@bbox, 
 				proj4string = x@proj4string,	
 				coords = x@coords[i, , drop = FALSE], grid = x@grid, 
 				grid.index = x@grid.index[i])
-			#x@coords = x@coords[i, , drop = FALSE]
-			#x@grid.index = x@grid.index[i]
-			#x@bbox = as.matrix(t(apply(x@coords, 2, range)))
-			#res = x
-		}
-		res
 	}
 )
 
@@ -208,16 +213,6 @@ setAs("SpatialPixels", "data.frame",
 setAs("SpatialGrid", "data.frame", 
 	function(from) as.data.frame.SpatialGrid(from))
 
-# uncommented 070122 RSB
-setAs("SpatialGrid", "SpatialPixels", function(from)
-	SpatialPixels(SpatialPoints(coordinates(from), from@proj4string))
-)
-# added EJP, 100521
-## outcommented 100607 as it breaks the ASDAR scripts in csdacm.R
-#setAs("SpatialGrid", "SpatialPoints", function(from)
-#	SpatialPoints(coordinates(from), from@proj4string)
-#)
-
 print.SpatialPixels = function(x, ...) {
 	cat("Object of class SpatialPixels\n")
 	print(summary(x@grid))
@@ -227,7 +222,6 @@ print.SpatialPixels = function(x, ...) {
 		"Coordinate Reference System (CRS) arguments:", 
 		proj4string(x))), collapse="\n")
 	cat(pst, "\n")
-#	print(as(x, "SpatialPoints"))
 	invisible(x)
 }
 setMethod("show", "SpatialPixels", function(object) print.SpatialPixels(object))
@@ -241,7 +235,6 @@ print.SpatialGrid = function(x, ...) {
 		"Coordinate Reference System (CRS) arguments:", 
 		proj4string(x))), collapse="\n")
 	cat(pst, "\n")
-#	print(as(x, "SpatialPoints"))
 	invisible(x)
 }
 setMethod("show", "SpatialGrid", function(object) print.SpatialGrid(object))
@@ -261,10 +254,6 @@ as.SpatialPolygons.SpatialPixels <- function(obj) {
 		yi <- obj_crds[i,2]
 		x <- c(xi-cS2x, xi-cS2x, xi+cS2x, xi+cS2x, xi-cS2x)
 		y <- c(yi-cS2y, yi+cS2y, yi+cS2y, yi-cS2y, yi-cS2y)
-#                x <- c(rep(xi-cS2x, 10), seq(xi-cS2x, xi+cS2x, length.out=10),
-#                    rep(xi+cS2x, 10), seq(xi+cS2x, xi-cS2x, length.out=10))
-#                y <- c(seq(yi-cS2y, yi+cS2y, length.out=10), rep(yi+cS2y, 10),
-#                    seq(yi+cS2y, yi-cS2y, length.out=10), rep(yi-cS2y, 10))
 		Srl[[i]] <- Polygons(list(Polygon(coords=cbind(x, y))), ID=IDs[i])
 	}
 	res <- SpatialPolygons(Srl, proj4string=CRS(proj4string(obj)))
@@ -285,5 +274,23 @@ IDvaluesSpatialPixels <- function(obj) {
 length.SpatialPixels = function(x) { nrow(x@coords) }
 length.SpatialGrid = function(x) { .NumberOfCells(x@grid) }
 
-setAs("SpatialGrid", "SpatialPolygons", function(from)
-	as.SpatialPolygons.GridTopology(from@grid))
+setAs("SpatialGrid", "SpatialPolygons", function(from) {
+		ret = as.SpatialPolygons.GridTopology(from@grid)
+		proj4string(ret) = proj4string(from)
+		ret
+	}
+)
+setMethod("coordnames", signature(x = "SpatialGrid"),
+	function(x) dimnames(bbox(x))[[1]])
+
+setReplaceMethod("coordnames", signature(x = "SpatialGrid", 
+	value =  "character"),
+    function(x, value) {
+		dimnames(x@bbox)[[1]] = value
+		coordnames(x@grid) = value
+		x
+	}
+)
+
+setAs("SpatialGrid", "GridTopology", function(from) getGridTopology(from))
+setAs("SpatialPixels", "GridTopology", function(from) getGridTopology(from))
