@@ -21,22 +21,23 @@
 .checkNumericCoerce2double = function(obj) {
 	if (any(!unlist(lapply(obj, is.numeric))))
 		stop("cannot retrieve coordinates from non-numeric elements")
-	#lapply(obj, function(x) { if(!is.numeric(x)) 
-	#	stop("cannot retrieve coordinates from non-numeric elements") })
-	lapply(obj, as.double)
+	fin_check <- sapply(obj, function(x) all(is.finite(x)))
+        if (!all(fin_check)) stop("non-finite coordinates")
+        lapply(obj, as.double)
 }
 
 setMethod("coordinates", "list", function(obj)
-		do.call("cbind", .checkNumericCoerce2double(as.data.frame(obj))))
+		do.call(cbind, .checkNumericCoerce2double(as.data.frame(obj))))
 setMethod("coordinates", "data.frame", function(obj)
-		do.call("cbind", .checkNumericCoerce2double(obj)))
+		do.call(cbind, .checkNumericCoerce2double(obj)))
 setMethod("coordinates", "matrix", 
 	function(obj) {
 		if (!is.numeric(obj))
 			stop("cannot derive coordinates from non-numeric matrix")
+                if (any(!is.finite(obj)))
+                    stop("non-finite coordinates")
 		dn = dimnames(obj)
 		dd = dim(obj)
-#		obj = apply(obj, 2, as.double)
 		storage.mode(obj) <- "double"
 		dim(obj) = dd
 		dimnames(obj) = dn
@@ -44,13 +45,14 @@ setMethod("coordinates", "matrix",
 	}
 )
 
-asWKTSpatialPoints = function(x, digits = 6) {
+asWKTSpatialPoints = function(x, digits = getOption("digits")) {
 	data.frame(geometry = paste("POINT(",unlist(lapply(data.frame(
 		t(signif(coordinates(x),digits = digits))),
 		paste, collapse=" ")),")",sep=""))
 }
 
-"print.SpatialPoints" <- function(x, ..., digits = 6, asWKT = .asWKT)
+"print.SpatialPoints" <- function(x, ..., digits = getOption("digits"), 
+	asWKT = .asWKT)
 {
 	cat("SpatialPoints:\n")
 	if (asWKT) 
@@ -84,16 +86,11 @@ as.data.frame.SpatialPoints = function(x, row.names, optional, ...) data.frame(x
 
 setAs("SpatialPoints", "data.frame", function(from) as.data.frame(from))
 
-subset.SpatialPoints <- function(x, subset, select, drop = FALSE, ...) {
-	if (!missing(select) && (length(select) < 2)) 
-		stop("selecting too few coordinate columns")
-	res <- SpatialPoints(subset(coordinates(x), subset, select, 
-		drop = drop), proj4string = proj4string(x))
-	res
-}
-
 row.names.SpatialPoints <- function(x) {
-    dimnames(slot(x, "coords"))[[1]]
+    ret = dimnames(slot(x, "coords"))[[1]]
+	if (is.null(ret))
+		ret = seq_len(nrow(slot(x, "coords")))
+	ret
 }
 
 "row.names<-.SpatialPoints" <- function(x, value) {
@@ -101,17 +98,18 @@ row.names.SpatialPoints <- function(x) {
 	x
 }
 
-#"[.SpatialPoints" =  function(x, i, j, ..., drop = T) {
 setMethod("[", "SpatialPoints", function(x, i, j, ..., drop = TRUE) {
 	if (!missing(j))
 		warning("j index ignored")
-	drop = FALSE
+	if (is.character(i))
+		i <- match(i, row.names(x))
+	else if (is(i, "Spatial"))
+		i = !is.na(over(x, geometry(i)))
 	if (any(is.na(i)))
 		stop("NAs not permitted in row index")
-#	SpatialPoints(coords=x@coords[i, , drop=drop], 
-#		proj4string = CRS(proj4string(x)))
 	x@coords = x@coords[i, , drop = FALSE]
-	x@bbox = .bboxCoords(x@coords)
+	if (drop)
+		x@bbox = .bboxCoords(x@coords)
 	x
 })
 
@@ -127,3 +125,13 @@ setReplaceMethod("coordnames", signature(x = "SpatialPoints", value = "character
 )
 
 length.SpatialPoints = function(x) { nrow(x@coords) }
+
+setMethod("$", "SpatialPoints", 
+	function(x, name) {
+		if (name %in% coordnames(x))
+			return(x@coords[,name])
+		if (!("data" %in% slotNames(x)))
+			stop("no $ method for object without attributes")
+		x@data[[name]]
+	}
+)
